@@ -1,9 +1,20 @@
 # -*- coding: utf-8 -*-
 """
     domonic.terminal
-    ~~~~~
+    ~~~~~~~~~~~~~~~~
     - call command line functions in python 3
 """
+import subprocess
+import os
+
+from domonic.javascript import window
+
+
+class TerminalException(Exception):
+    def __init__(self, error, message="An error message was recieved from terminal"):
+        print(error, message)
+        self.message = message
+        super().__init__(self.message)
 
 
 class command():
@@ -11,62 +22,115 @@ class command():
     def __init__(self, *args, **kwargs):
         self.args = args
         self.kwargs = kwargs
-
         self.params = ''.join([each.__str__() for each in args])
-        # self.attributes = ''.join([ ''' %s="%s"''' % (key.split('_',1)[1], value) for key, value in kwargs.items()])
+        self.result = ""
 
-        import subprocess
-        cmd = f"{self.name} {self.params}"
-        cmd = cmd.strip()
+        if not hasattr(self, 'first_run'):
+            self.first_run = True
+        else:
+            self.first_run = False
 
-        # print('running: ', cmd)
+        self.run_command()
 
-        returned_output = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
-        # print(returned_output.decode("utf-8"))
-        self.result = returned_output.decode("utf-8")
-
-    def __str__(self):
-        # return f"{self.name} {self.params}"
+    def __call__(self, *args, **kwargs):
+        self.run_command()
         return self.result
 
-    # def __repr__(self):
-    #     return f"<{self.name}{self.attributes}>{self.content}</{self.name}>"
+    def run_command(self):
+        if hasattr(self, 'wait'):
+            self.has_wait = True
+            if self.first_run is True:
+                self.first_run = False
+                return
+        else:
+            self.has_wait = False
 
-    # def __repr__(self):
-        # return repr([self.args])
-        # return repr(self.result)
+        if hasattr(self, 'iterable'):
+            if self.first_run is True:
+                self.first_run = False
+                return
+
+        try:
+            cmd = f"{self.name} {self.params}"
+            cmd = cmd.strip()
+
+            '''
+            # TODO - after doing all this. think i may have decided to go for 2 base commands instead
+            # one iterable and one normal. its still hidden from user as just a different inhereted command
+            # but would keep both simpler and can have more types. so will have to refactor all this again
+            # for now this behaves how i want despite the _new_ hack and double call on this command
+            '''
+            if self.has_wait is not True:
+                returned_output = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
+                self.result = returned_output.decode("utf-8")
+            else:
+                self.result = 'PING FAIL'
+
+                def kill_switch(proc):
+                    proc.kill()
+                    return
+
+                # try:
+                proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+                intId = window.setInterval(3000, kill_switch, proc)
+                self.result = proc.stdout.readlines()
+                window.clearInterval(intId)
+                # except subprocess.TimeoutExpired as e:
+                # self.result = returned_output.decode("utf-8")
+                # print('process ran too long')
+
+        except subprocess.CalledProcessError as e:
+            print(e.output)
+            self.result = e.output
+            raise TerminalException(e, self.result)
+
+        return
+
+    def __new__(cls, *args, **kwargs):
+        instance = super().__new__(cls)
+        instance.__init__(*args, **kwargs)
+        if hasattr(instance, 'iterable'):
+            is_iterable = instance.iterable
+            if is_iterable:
+                return instance
+
+        return str(instance.result)
+
+    def __str__(self):
+        return str(self.result)
 
     def __getitem__(self, index):
-        # return self.args[index]
         return self.result.splitlines()[index]
 
-    # def __setitem__(self,key,value):
-        # self.args[key] = value
-        # print( self.args[key] )
-        # return self.result
+
+class cd(command):
+    ''' NOTE - does not run on terminal '''
+    # cd is pointless as session opens and closes
+    # so override run command and do something else
+    def run_command(self):
+        os.chdir(self.params)
 
 
-ls = type('ls', (command,), {'name': 'ls'})
-cd = type('cd', (command,), {'name': 'cd'})
+# tested --
+ls = type('ls', (command,), {'name': 'ls', 'iterable': True})
 rm = type('rm', (command,), {'name': 'rm'})
 ln = type('ln', (command,), {'name': 'ln'})
 mkdir = type('mkdir', (command,), {'name': 'mkdir'})
 rmdir = type('rmdir', (command,), {'name': 'rmdir'})
 touch = type('touch', (command,), {'name': 'touch'})
+df = type('df', (command,), {'name': 'df', 'iterable': True})
+du = type('du', (command,), {'name': 'du', 'iterable': True})
+ps = type('ps', (command,), {'name': 'ps', 'iterable': True})
+cat = type('cat', (command,), {'name': 'cat', 'iterable': True})
+# --
 
-df = type('df', (command,), {'name': 'df'})
-du = type('du', (command,), {'name': 'du'})
-ps = type('ps', (command,), {'name': 'ps'})
-
-cat = type('cat', (command,), {'name': 'cat'})
 mv = type('mv', (command,), {'name': 'mv'})
 cp = type('cp', (command,), {'name': 'cp'})
 rm = type('rm', (command,), {'name': 'rm'})
-
 chmod = type('chmod', (command,), {'name': 'chmod'})
 chown = type('chown', (command,), {'name': 'chown'})
 
-users = type('users', (command,), {'name': 'users'})
+users = type('users', (command,), {'name': 'users', 'iterable': True})
 useradd = type('useradd', (command,), {'name': 'useradd'})
 userdel = type('userdel', (command,), {'name': 'userdel'})
 groups = type('groups', (command,), {'name': 'groups'})
@@ -75,10 +139,9 @@ groupdel = type('groupdel', (command,), {'name': 'groupdel'})
 
 uniq = type('uniq', (command,), {'name': 'uniq'})
 sort = type('sort', (command,), {'name': 'sort'})
-dif = type('dif', (command,), {'name': 'dif'})
+diff = type('diff', (command,), {'name': 'diff'})
 # cmp = type('cmp', (command,), {'name': 'cmp'})
 
-
 ssh = type('ssh', (command,), {'name': 'ssh'})
 scp = type('scp', (command,), {'name': 'scp'})
 
@@ -86,34 +149,38 @@ more = type('more', (command,), {'name': 'more'})
 less = type('less', (command,), {'name': 'less'})
 # vi = type('vi', (command,), {'name': 'vi'})
 # emacs = type('emacs', (command,), {'name': 'emacs'})
-head = type('head', (command,), {'name': 'head'})
-tail = type('tail', (command,), {'name': 'tail'})
+head = type('head', (command,), {'name': 'head', 'iterable': True})
+tail = type('tail', (command,), {'name': 'tail', 'iterable': True})
 
 uptime = type('uptime', (command,), {'name': 'uptime'})
 bash = type('bash', (command,), {'name': 'bash'})
-
-'''
-def Atag(self,*args,**kwargs):
-    tag.__init__(self,*args,**kwargs)
-    URL.__init__(self, url=kwargs['_href'])
-
-def __update__(self):
-    URL.__update__(self)
-    tag.__init__(self,_href=self.href)
-    
-a = type('a', (tag, Element, URL), {'name': 'a', '__init__':Atag, '__update__':__update__})
-'''
-
-# class ls():
-#     def __init__(self,content=""):
-#         self.content = content
-#     def __str__(self):
-        # return f"<!-- {self.content} -->"
-
-
+shutdown = type('shutdown', (command,), {'name': 'shutdown'})
+reboot = type('reboot', (command,), {'name': 'reboot'})
+# open = type('open', (command,), {'name': 'open'})
+killall = type('killall', (command,), {'name': 'killall'})
+mkfile = type('mkfile', (command,), {'name': 'mkfile'})
+ifconfig = type('ifconfig', (command,), {'name': 'ifconfig', 'iterable': True})
+ipconfig = type('ipconfig', (command,), {'name': 'ipconfig', 'iterable': True})
+finger = type('finger', (command,), {'name': 'finger'})
+passwd = type('passwd', (command,), {'name': 'passwd'})
+whoami = type('whoami', (command,), {'name': 'whoami'})
 # help = type('help', (command,), {'name': 'help'})
 
-ping = type('ping', (command,), {'name': 'ping'}) # < TODO - need to stream feedback
+
+class history(command):
+    def run_command(self):
+        self.iterable = True
+        try:
+            # history is empty for non interactive sessions so read file
+            from os.path import join, expanduser
+            with open(join(expanduser('~'), '.bash_history'), 'r') as f:
+                self.result = ''.join(f.readlines())
+        except Exception as e:
+            print('failed to get history:', e)
+            self.result = ''
+
+
+ping = type('ping', (command,), {'name': 'ping', 'wait': True, 'iterable': True})  # < TODO - need to stream feedback
 man = type('man', (command,), {'name': 'man'})
 find = type('find', (command,), {'name': 'find'})
 awk = type('awk', (command,), {'name': 'awk'})
@@ -127,7 +194,7 @@ gzip = type('gzip', (command,), {'name': 'gzip'})
 cron = type('cron', (command,), {'name': 'cron'})
 crontab = type('crontab', (command,), {'name': 'crontab'})
 xargs = type('xargs', (command,), {'name': 'xargs'})
-nautilus = type('nautilus', (command,), {'name': 'nautilus'}) # Linux only?
+nautilus = type('nautilus', (command,), {'name': 'nautilus'})  # Linux only?
 date = type('date', (command,), {'name': 'date'})
 cal = type('cal', (command,), {'name': 'cal'})
 bc = type('bc', (command,), {'name': 'bc'})
@@ -143,7 +210,7 @@ nohup = type('nohup', (command,), {'name': 'nohup'})
 python = type('python', (command,), {'name': 'python'})
 npm = type('npm', (command,), {'name': 'npm'})
 cowsay = type('cowsay', (command,), {'name': 'cowsay'})
-pip = type('pip', (command,), {'name': 'pip'})  # TODO - change to 'python3 -m pip'?
+pip = type('pip', (command,), {'name': 'pip'})  # TODO - change to 'python3 -m pip'? # TODO - both iterable and not
 say = type('say', (command,), {'name': 'say'})  # Mac only?
 gcc = type('gcc', (command,), {'name': 'gcc'})
 jq = type('jq', (command,), {'name': 'jq'})
@@ -151,12 +218,16 @@ curl = type('curl', (command,), {'name': 'curl'})
 ffmpeg = type('ffmpeg', (command,), {'name': 'ffmpeg'})
 convert = type('convert', (command,), {'name': 'convert'})  # imagemagik
 figlet = type('figlet', (command,), {'name': 'figlet'})
-banner = type('banner', (command,), {'name': 'banner'})  # imagemagik
+banner = type('banner', (command,), {'name': 'banner'})
+# imagemagik
+# asciiview
+# toilet
+# figlet
 # youtube-dl = type('youtube-dl', (command,), {'name': 'youtube-dl'}) # TODO - illegal chars
 # tree = type('tree', (command,), {'name': 'tree'})
 # vim = type('vim', (command,), {'name': 'vim'})
 # mail = type('mail', (command,), {'name': 'mail'})
-
+# vncserver = type('vncserver', (command,), {'name': 'vncserver'})
 
 # https://en.wikipedia.org/wiki/List_of_Unix_commands
 # admin = type('admin', (command,), {'name': 'admin'})
@@ -171,7 +242,6 @@ cflow = type('cflow', (command,), {'name': 'cflow'})
 chgrp = type('chgrp', (command,), {'name': 'chgrp'})
 cksum = type('cksum', (command,), {'name': 'cksum'})
 comm = type('comm', (command,), {'name': 'comm'})
-command = type('command', (command,), {'name': 'command'})
 compress = type('compress', (command,), {'name': 'compress'})
 csplit = type('csplit', (command,), {'name': 'csplit'})
 ctags = type('ctags', (command,), {'name': 'ctags'})
@@ -187,7 +257,6 @@ expr = type('expr', (command,), {'name': 'expr'})
 fc = type('fc', (command,), {'name': 'fc'})
 fg = type('fg', (command,), {'name': 'fg'})
 file = type('file', (command,), {'name': 'file'})
-find = type('find', (command,), {'name': 'find'})
 fold = type('fold', (command,), {'name': 'fold'})
 fort77 = type('fort77', (command,), {'name': 'fort77'})
 fuser = type('fuser', (command,), {'name': 'fuser'})
@@ -195,12 +264,10 @@ gencat = type('gencat', (command,), {'name': 'gencat'})
 get = type('get', (command,), {'name': 'get'})
 getconf = type('getconf', (command,), {'name': 'getconf'})
 getopts = type('getopts', (command,), {'name': 'getopts'})
-head = type('head', (command,), {'name': 'head'})
 iconv = type('iconv', (command,), {'name': 'iconv'})
 # id = type('id', (command,), {'name': 'id'})
 ipcrm = type('ipcrm', (command,), {'name': 'ipcrm'})
 ipcs = type('ipcs', (command,), {'name': 'ipcs'})
-jobs = type('jobs', (command,), {'name': 'jobs'})
 join = type('join', (command,), {'name': 'join'})
 lex = type('lex', (command,), {'name': 'lex'})
 link = type('link', (command,), {'name': 'link'})
@@ -212,7 +279,6 @@ lp = type('lp', (command,), {'name': 'lp'})
 m4 = type('m4', (command,), {'name': 'm4'})
 mailx = type('mailx', (command,), {'name': 'mailx'})
 make = type('make', (command,), {'name': 'make'})
-man = type('man', (command,), {'name': 'man'})
 mesg = type('mesg', (command,), {'name': 'mesg'})
 mkdir = type('mkdir', (command,), {'name': 'mkdir'})
 mkfifo = type('mkfifo', (command,), {'name': 'mkfifo'})
@@ -242,7 +308,6 @@ qsub = type('qsub', (command,), {'name': 'qsub'})
 read = type('read', (command,), {'name': 'read'})
 renice = type('renice', (command,), {'name': 'renice'})
 rmdel = type('rmdel', (command,), {'name': 'rmdel'})
-rmdir = type('rmdir', (command,), {'name': 'rmdir'})
 sact = type('sact', (command,), {'name': 'sact'})
 sccs = type('sccs', (command,), {'name': 'sccs'})
 sed = type('sed', (command,), {'name': 'sed'})
@@ -253,7 +318,6 @@ strings = type('strings', (command,), {'name': 'strings'})
 strip = type('strip', (command,), {'name': 'strip'})
 stty = type('stty', (command,), {'name': 'stty'})
 tabs = type('tabs', (command,), {'name': 'tabs'})
-tail = type('tail', (command,), {'name': 'tail'})
 talk = type('talk', (command,), {'name': 'talk'})
 tee = type('tee', (command,), {'name': 'tee'})
 test = type('test', (command,), {'name': 'test'})
@@ -281,7 +345,6 @@ write = type('write', (command,), {'name': 'write'})
 yacc = type('yacc', (command,), {'name': 'yacc'})
 zcat = type('zcat', (command,), {'name': 'zcat'})
 
-
 # bash builtins
 alias = type('alias', (command,), {'name': 'alias'})
 bg = type('bg', (command,), {'name': 'bg'})
@@ -289,7 +352,7 @@ bind = type('bind', (command,), {'name': 'bind'})
 # break = type('break', (command,), {'name': 'break'})
 builtin = type('builtin', (command,), {'name': 'builtin'})
 caller = type('caller', (command,), {'name': 'caller'})
-command = type('command', (command,), {'name': 'command'})
+# command = type('command', (command,), {'name': 'command'})
 compgen = type('compgen', (command,), {'name': 'compgen'})
 complete = type('complete', (command,), {'name': 'complete'})
 compopt = type('compopt', (command,), {'name': 'compopt'})
@@ -309,7 +372,7 @@ fg = type('fg', (command,), {'name': 'fg'})
 getopts = type('getopts', (command,), {'name': 'getopts'})
 # hash = type('hash', (command,), {'name': 'hash'})
 # help = type('help', (command,), {'name': 'help'})
-history = type('history', (command,), {'name': 'history'}) # < TODO - NOT WORKING. try with stream method after doing 'ping'
+
 jobs = type('jobs', (command,), {'name': 'jobs'})
 kill = type('kill', (command,), {'name': 'kill'})
 let = type('let', (command,), {'name': 'let'})
@@ -342,4 +405,4 @@ unset = type('unset', (command,), {'name': 'unset'})
 wait = type('wait', (command,), {'name': 'wait'})
 
 # https://ss64.com/bash/
-# PIPE etc? 
+# PIPE etc?
